import random

import abjad
import pytest

import auxjad


def test_ListLooper_01():
    input_list = ['A', 'B', 'C', 'D', 'E', 'F']
    looper = auxjad.ListLooper(input_list, window_size=3)
    assert format(looper) == "['A', 'B', 'C', 'D', 'E', 'F']"
    assert looper() == ['A', 'B', 'C']
    assert looper() == ['B', 'C', 'D']
    assert looper.current_window == ['B', 'C', 'D']


def test_ListLooper_02():
    input_list = ['A', 'B', 'C', 'D', 'E', 'F']
    looper = auxjad.ListLooper(input_list,
                               window_size=3,
                               step_size=1,
                               max_steps=2,
                               repetition_chance=0.25,
                               forward_bias=0.2,
                               head_position=0,
                               end_with_max_n_elements=True,
                               process_on_first_call=True,
                               )
    assert looper.window_size == 3
    assert looper.step_size == 1
    assert looper.max_steps == 2
    assert looper.repetition_chance == 0.25
    assert looper.forward_bias == 0.2
    assert looper.head_position == 0
    assert looper.end_with_max_n_elements
    assert looper.process_on_first_call
    looper.window_size = 2
    looper.step_size = 2
    looper.max_steps = 3
    looper.repetition_chance = 0.1
    looper.forward_bias = 0.8
    looper.head_position = 2
    looper.end_with_max_n_elements = False
    looper.process_on_first_call = False
    assert looper.window_size == 2
    assert looper.step_size == 2
    assert looper.max_steps == 3
    assert looper.repetition_chance == 0.1
    assert looper.forward_bias == 0.8
    assert looper.head_position == 2
    assert not looper.end_with_max_n_elements
    assert not looper.process_on_first_call


def test_ListLooper_03():
    input_list = ['A', 'B', 'C', 'D', 'E', 'F']
    looper = auxjad.ListLooper(input_list, window_size=3)
    assert looper.head_position == 0
    looper()
    assert looper.head_position == 0
    looper()
    assert looper.head_position == 1
    looper()
    assert looper.head_position == 2


def test_ListLooper_04():
    input_list = ['A', 'B', 'C', 'D', 'E', 'F']
    looper = auxjad.ListLooper(input_list, window_size=3)
    assert len(looper) == 6


def test_ListLooper_05():
    input_list = ['A', 'B', 'C', 'D']
    looper = auxjad.ListLooper(input_list, window_size=3)
    assert looper.output_all() == ['A', 'B', 'C', 'B', 'C', 'D', 'C', 'D', 'D']


def test_ListLooper_06():
    input_list = ['A', 'B', 'C', 'D', 'E', 'F']
    looper = auxjad.ListLooper(input_list, window_size=3)
    assert looper() == ['A', 'B', 'C']
    looper.window_size = 4
    assert looper() == ['B', 'C', 'D', 'E']


def test_ListLooper_07():
    input_list = [123, 'foo', (3, 4), 3.14]
    looper = auxjad.ListLooper(input_list, window_size=3)
    assert looper() == [123, 'foo', (3, 4)]


def test_ListLooper_08():
    input_list = [
        abjad.Container(r"c'4 d'4 e'4 f'4"),
        abjad.Container(r"fs'1"),
        abjad.Container(r"r2 bf2"),
        abjad.Container(r"c''2. r4"),
    ]
    looper = auxjad.ListLooper(input_list, window_size=3)
    staff = abjad.Staff()
    for element in looper.output_all():
        staff.append(element)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            {
                c'4
                d'4
                e'4
                f'4
            }
            {
                fs'1
            }
            {
                r2
                bf2
            }
            {
                fs'1
            }
            {
                r2
                bf2
            }
            {
                c''2.
                r4
            }
            {
                r2
                bf2
            }
            {
                c''2.
                r4
            }
            {
                c''2.
                r4
            }
        }
        """
    )


def test_ListLooper_09():
    input_list = ['A', 'B', 'C', 'D']
    looper = auxjad.ListLooper(input_list, window_size=3)
    assert looper.__next__() == ['A', 'B', 'C']
    assert looper.__next__() == ['B', 'C', 'D']
    assert looper.__next__() == ['C', 'D']
    assert looper.__next__() == ['D']
    with pytest.raises(StopIteration):
        notes = looper.__next__()  # noqa: F841


def test_ListLooper_10():
    wrong_type_input = 'foo'
    input_list = ['A', 'B', 'C', 'D']
    with pytest.raises(TypeError):
        looper = auxjad.ListLooper(wrong_type_input,  # noqa: F841
                                   window_size=3,
                                   )
        looper = auxjad.ListLooper(input_list,  # noqa: F841
                                   window_size='foobar',
                                   )
        looper = auxjad.ListLooper(input_list,  # noqa: F841
                                   window_size=3,
                                   step_size='foobar',
                                   )
        looper = auxjad.ListLooper(input_list,  # noqa: F841
                                   window_size=3,
                                   max_steps='foobar',
                                   )
        looper = auxjad.ListLooper(input_list,  # noqa: F841
                                   window_size=3,
                                   repetition_chance='foobar',
                                   )
        looper = auxjad.ListLooper(input_list,  # noqa: F841
                                   window_size=3,
                                   head_position='foobar',
                                   )
    with pytest.raises(ValueError):
        looper = auxjad.ListLooper(input_list, window_size=-1)  # noqa: F841
        looper = auxjad.ListLooper(input_list,  # noqa: F841
                                   window_size=3,
                                   step_size=-1)
        looper = auxjad.ListLooper(input_list,  # noqa: F841
                                   window_size=3,
                                   step_size=100)
        looper = auxjad.ListLooper(input_list,  # noqa: F841
                                   window_size=3,
                                   max_steps=-1)
        looper = auxjad.ListLooper(input_list,  # noqa: F841
                                   window_size=3,
                                   repetition_chance=-0.3)
        looper = auxjad.ListLooper(input_list,  # noqa: F841
                                   window_size=3,
                                   repetition_chance=1.4)
        looper = auxjad.ListLooper(input_list,  # noqa: F841
                                   window_size=3,
                                   head_position=-1)
        looper = auxjad.ListLooper(input_list,  # noqa: F841
                                   window_size=3,
                                   head_position=100)


def test_ListLooper_11():
    input_list = ['A', 'B', 'C', 'D']
    looper = auxjad.ListLooper(input_list, window_size=3)
    assert looper.output_n(2) == ['A', 'B', 'C', 'B', 'C', 'D']


def test_ListLooper_12():
    input_list = ['A', 'B', 'C', 'D']
    looper = auxjad.ListLooper(input_list, window_size=3)
    with pytest.raises(RuntimeError):
        notes = looper.output_n(100)  # noqa: F841


def test_ListLooper_13():
    input_list = ['A', 'B', 'C', 'D']
    looper = auxjad.ListLooper(input_list,
                               window_size=2,
                               head_position=2,
                               forward_bias=0.0,
                               )
    assert looper.output_all() == ['C', 'D', 'B', 'C', 'A', 'B']


def test_ListLooper_14():
    input_list = ['A', 'B', 'C', 'D']
    looper = auxjad.ListLooper(input_list,
                               window_size=2,
                               head_position=0,
                               forward_bias=0.0,
                               )
    assert looper.output_all() == ['A', 'B']


def test_ListLooper_15():
    input_list = ['A', 'B', 'C', 'D', 'E', 'F']
    looper = auxjad.ListLooper(input_list,
                               window_size=3,
                               process_on_first_call=True,
                               )
    assert looper() == ['B', 'C', 'D']


def test_ListLooper_16():
    input_list = ['A', 'B', 'C', 'D', 'E', 'F']
    looper = auxjad.ListLooper(input_list,
                               window_size=3,
                               )
    assert looper.contents == ['A', 'B', 'C', 'D', 'E', 'F']
    assert looper() == ['A', 'B', 'C']
    assert looper() == ['B', 'C', 'D']
    looper.contents = [0, 1, 2, 3, 4]
    assert looper.contents == [0, 1, 2, 3, 4]
    assert looper() == [1, 2, 3]
    looper.head_position = 0
    assert looper() == [0, 1, 2]


def test_ListLooper_17():
    random.seed(85162)
    input_list = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H']
    looper = auxjad.ListLooper(input_list,
                               window_size=2,
                               head_position=4,
                               forward_bias=0.5,
                               )
    assert looper.output_n(4) == ['E', 'F', 'D', 'E', 'C', 'D', 'B', 'C']


def test_ListLooper_18():
    random.seed(70013)
    input_list = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H']
    looper = auxjad.ListLooper(input_list,
                               window_size=2,
                               head_position=2,
                               max_steps=4,
                               )
    assert looper.output_n(4) == ['C', 'D', 'D', 'E', 'E', 'F', 'H']


def test_ListLooper_19():
    input_list = ['A', 'B', 'C', 'D']
    looper = auxjad.ListLooper(input_list, window_size=3)
    assert looper.output_all() == ['A', 'B', 'C', 'B', 'C', 'D', 'C', 'D', 'D']
    input_list = ['A', 'B', 'C', 'D']
    looper = auxjad.ListLooper(input_list,
                               window_size=3,
                               end_with_max_n_elements=True,
                               )
    assert looper.output_all() == ['A', 'B', 'C', 'B', 'C', 'D']



import random

import pytest

import auxjad


def test_TenneySelector_01():
    random.seed(43714)
    selector = auxjad.TenneySelector(['A', 'B', 'C', 'D', 'E', 'F'])
    assert selector.contents == ['A', 'B', 'C', 'D', 'E', 'F']
    assert format(selector) == "['A', 'B', 'C', 'D', 'E', 'F']"
    assert selector.curvature == 1.0
    assert selector.weights == [1.0, 1.0, 1.0, 1.0, 1.0, 1.0]
    assert selector.probabilities == [1.0, 1.0, 1.0, 1.0, 1.0, 1.0]
    assert selector.previous_index is None
    assert selector.previous_result is None
    assert len(selector) == 6
    result = ''
    for _ in range(30):
        result += selector()
    assert result == 'BDAFCBEFCDFDBAEDFCABDEABCDEBFE'
    assert selector.previous_index == 4
    assert selector.previous_result == 'E'
    with pytest.raises(AttributeError):
        selector.previous_index = 3
        selector.previous_result = 'C'
    assert selector.weights == [1.0, 1.0, 1.0, 1.0, 1.0, 1.0]
    assert selector.probabilities == [7.0, 2.0, 5.0, 4.0, 0.0, 1.0]


def test_TenneySelector_02():
    random.seed(37248)
    selector = auxjad.TenneySelector(['A', 'B', 'C', 'D', 'E', 'F'],
                                     curvature=0.001,
                                     )
    result = ''
    for _ in range(30):
        result += selector()
    assert result == 'CDCBFECBCEBCFCEDFACDBADCADCDFE'
    assert selector.probabilities == [1.0016107337527294,
                                      1.002199640244188,
                                      1.001099215984204,
                                      1.0006933874625807,
                                      0.0,
                                      1.0,
                                      ]


def test_TenneySelector_03():
    random.seed(76231)
    selector = auxjad.TenneySelector(['A', 'B', 'C', 'D', 'E', 'F'],
                                     curvature=15.2,
                                     )
    result = ''
    for _ in range(30):
        result += selector()
    assert result == 'DFAECBDFAECBDFAECBDFAECBDFAECB'
    assert selector.probabilities == [17874877.39956566,
                                      0.0,
                                      1.0,
                                      42106007735.02238,
                                      37640.547696542824,
                                      1416810830.8957152,
                                      ]


def test_TenneySelector_04():
    random.seed(14625)
    selector = auxjad.TenneySelector(
        ['A', 'B', 'C', 'D', 'E', 'F'],
        weights=[1.0, 1.0, 5.0, 5.0, 10.0, 20.0],
    )
    assert selector.weights == [1.0, 1.0, 5.0, 5.0, 10.0, 20.0]
    assert selector.probabilities == [1.0, 1.0, 5.0, 5.0, 10.0, 20.0]
    result = ''
    for _ in range(30):
        result += selector()
    assert result == 'FBEFECFDEADFEDFEDBFECDAFCEDCFE'
    assert selector.weights == [1.0, 1.0, 5.0, 5.0, 10.0, 20.0]
    assert selector.probabilities == [7.0, 12.0, 10.0, 15.0, 0.0, 20.0]


def test_TenneySelector_05():
    random.seed(21169)
    selector = auxjad.TenneySelector(['A', 'B', 'C', 'D', 'E', 'F'])
    for _ in range(30):
        selector()
    assert selector.probabilities == [3.0, 2.0, 1.0, 7.0, 5.0, 0.0]
    assert selector[2] == 'C'
    assert selector[1:4] == ['B', 'C', 'D']
    selector[2] = 'foo'
    assert selector[:] == ['A', 'B', 'foo', 'D', 'E', 'F']
    selector[:] = ['foo', 'bar', 'X', 'Y', 'Z', '...']
    assert selector.contents == ['foo', 'bar', 'X', 'Y', 'Z', '...']
    assert selector.probabilities == [3.0, 2.0, 1.0, 7.0, 5.0, 0.0]
    del selector[0:2]
    assert selector.contents == ['X', 'Y', 'Z', '...']
    assert selector.probabilities == [1.0, 7.0, 5.0, 0.0]
    assert 'X' in selector
    assert 'A' not in selector


def test_TenneySelector_06():
    random.seed(54267)
    selector = auxjad.TenneySelector(
        ['A', 'B', 'C', 'D', 'E', 'F'],
        weights=[1.0, 1.0, 5.0, 5.0, 10.0, 20.0],
    )
    for _ in range(30):
        selector()
    assert len(selector) == 6
    assert selector.contents == ['A', 'B', 'C', 'D', 'E', 'F']
    assert selector.weights == [1.0, 1.0, 5.0, 5.0, 10.0, 20.0]
    assert selector.probabilities == [8.0, 2.0, 5.0, 15.0, 50.0, 0.0]

    selector.contents = [2, 4, 6, 8]
    assert len(selector) == 4
    assert selector.contents == [2, 4, 6, 8]
    assert selector.weights == [1.0, 1.0, 1.0, 1.0]
    assert selector.probabilities == [1.0, 1.0, 1.0, 1.0]

    selector.weights = [1.2, 3.0, 2.5, 1.3]
    assert selector.weights == [1.2, 3.0, 2.5, 1.3]


def test_TenneySelector_07():
    selector = auxjad.TenneySelector(['A', 'B', 'C', 'D', 'E', 'F'])
    assert selector.curvature == 1.0
    selector.curvature = 0.25
    assert selector.curvature == 0.25


def test_TenneySelector_08():
    random.seed(99651)
    selector = auxjad.TenneySelector(['A', 'B', 'C', 'D', 'E', 'F'])
    for _ in range(30):
        selector()
    assert selector.probabilities == [4.0, 3.0, 1.0, 0.0, 5.0, 2.0]
    selector.reset_probabilities()
    assert selector.probabilities == [1.0, 1.0, 1.0, 1.0, 1.0, 1.0]


def test_TenneySelector_09():
    random.seed(12387)
    selector = auxjad.TenneySelector(['A', 'B', 'C', 'D', 'E', 'F'])
    result = ''
    result += selector.__next__()
    result += selector.__next__()
    result += selector.__next__()
    result += next(selector)
    result += next(selector)
    result += next(selector)
    assert result == 'DBDFAE'


def test_TenneySelector_10():
    selector = auxjad.TenneySelector(
        ['A', 'B', 'C', 'D', 'E', 'F'],
        weights=[1.0, 1.0, 5.0, 5.0, 10.0, 20.0],
    )
    assert selector.weights == [1.0, 1.0, 5.0, 5.0, 10.0, 20.0]
    selector.weights = None
    assert selector.weights == [1.0, 1.0, 1.0, 1.0, 1.0, 1.0]



import abjad

import auxjad


def test_leaves_are_tieable_01():
    leaf1 = abjad.Note(r"c'4")
    leaf2 = abjad.Note(r"c'4")
    assert auxjad.get.leaves_are_tieable([leaf1, leaf2])


def test_leaves_are_tieable_02():
    leaf1 = abjad.Note(r"c'2.")
    leaf2 = abjad.Note(r"c'16")
    leaf3 = abjad.Note(r"f'''16")
    leaf4 = abjad.Note(r"r2.")
    assert auxjad.get.leaves_are_tieable([leaf1, leaf2])
    assert not auxjad.get.leaves_are_tieable([leaf1, leaf3])
    assert not auxjad.get.leaves_are_tieable([leaf2, leaf3])
    assert not auxjad.get.leaves_are_tieable([leaf3, leaf4])


def test_leaves_are_tieable_03():
    chord1 = abjad.Chord(r"<c' e' g'>4")
    chord2 = abjad.Chord(r"<c' e' g'>16")
    chord3 = abjad.Chord(r"<f''' fs'''>16")
    assert auxjad.get.leaves_are_tieable([chord1, chord2])
    assert not auxjad.get.leaves_are_tieable([chord1, chord3])
    assert not auxjad.get.leaves_are_tieable([chord2, chord3])


def test_leaves_are_tieable_04():
    chord1 = abjad.Chord(r"<c' e' g'>4")
    chord2 = abjad.Chord(r"<c' e' g' bf'>4")
    note = abjad.Note(r"c'4")
    assert auxjad.get.leaves_are_tieable([chord1, chord2])
    assert auxjad.get.leaves_are_tieable([chord2, note])


def test_leaves_are_tieable_05():
    container = abjad.Container(r"r4 <c' e'>4 <c' e'>2")
    assert auxjad.get.leaves_are_tieable([container[1], container[2]])


def test_leaves_are_tieable_06():
    leaf = abjad.Note(r"c'4")
    rest = abjad.Rest(r"r4")
    assert not auxjad.get.leaves_are_tieable([leaf, rest])


def test_leaves_are_tieable_07():
    staff = abjad.Staff(r"c'4 d'4 e'2 e'1")
    assert not auxjad.get.leaves_are_tieable(staff[:2])
    assert not auxjad.get.leaves_are_tieable(staff[1:3])
    assert auxjad.get.leaves_are_tieable(staff[2:4])


def test_leaves_are_tieable_08():
    staff = abjad.Staff(r"c'2 c'4. c'8")
    assert auxjad.get.leaves_are_tieable(staff[:])
    staff = abjad.Staff(r"c'2 c'4. d'8")
    assert not auxjad.get.leaves_are_tieable(staff[:])


def test_leaves_are_tieable_09():
    leaf1 = abjad.Note(r"c'4")
    leaf2 = abjad.Note(r"c'4")
    leaf3 = abjad.Note(r"c'2.")
    assert auxjad.get.leaves_are_tieable([leaf1, leaf2, leaf3])


def test_leaves_are_tieable_10():
    leaf = abjad.Note(r"c'2.")
    staff = abjad.Staff(r"c'4 ~ c'16")
    logical_tie = abjad.select(staff).logical_tie(0)
    assert auxjad.get.leaves_are_tieable([leaf, logical_tie])


def test_leaves_are_tieable_11():
    leaf1 = abjad.Note(r"c'2.")
    leaf2 = abjad.Note(r"c'16")
    leaf3 = abjad.Note(r"f'''16")
    assert abjad.get.leaves_are_tieable([leaf1, leaf2])
    assert not abjad.get.leaves_are_tieable([leaf1, leaf3])
    assert not abjad.get.leaves_are_tieable([leaf2, leaf3])


def test_leaves_are_tieable_12():
    leaf1 = abjad.Note(r"c'4")
    leaf2 = abjad.Chord(r"<c' e'>4")
    leaf3 = abjad.Note(r"c'2")
    assert auxjad.get.leaves_are_tieable([leaf1, leaf2])
    assert not auxjad.get.leaves_are_tieable([leaf1, leaf2],
                                             only_identical_pitches=True,
                                             )
    assert auxjad.get.leaves_are_tieable([leaf2, leaf3])
    assert not auxjad.get.leaves_are_tieable([leaf2, leaf3],
                                             only_identical_pitches=True,
                                             )


def test_leaves_are_tieable_13():
    chord1 = abjad.Chord(r"<c' e' g'>4")
    chord2 = abjad.Chord(r"<c' e' g' bf'>4")
    chord3 = abjad.Chord(r"<c' e' fs'>4")
    assert auxjad.get.leaves_are_tieable([chord1, chord2])
    assert not auxjad.get.leaves_are_tieable([chord1, chord2],
                                             only_identical_pitches=True,
                                             )
    assert auxjad.get.leaves_are_tieable([chord2, chord3])
    assert not auxjad.get.leaves_are_tieable([chord2, chord3],
                                             only_identical_pitches=True,
                                             )


def test_leaves_are_tieable_14():
    staff = abjad.Staff(r"c'2 c'4. <c' e'>8")
    assert auxjad.get.leaves_are_tieable(staff[:])
    assert not auxjad.get.leaves_are_tieable(staff[:],
                                             only_identical_pitches=True,
                                             )



import random

import abjad
import pytest

import auxjad


def test_LeafLooper_01():
    container = abjad.Container(r"c'4 d'2 e'4 f'2 ~ f'8 g'4.")
    looper = auxjad.LeafLooper(container, window_size=3)
    assert abjad.lilypond(looper) == abjad.String.normalize(
        r"""
        {
            c'4
            d'2
            e'4
            f'2
            ~
            f'8
            g'4.
        }
        """
    )
    notes = looper()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            d'2
            e'4
        }
        """
    )
    notes = looper()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 11/8
            d'2
            e'8
            ~
            e'8
            f'8
            ~
            f'2
        }
        """
    )
    notes = looper.current_window
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 11/8
            d'2
            e'8
            ~
            e'8
            f'8
            ~
            f'2
        }
        """
    )


def test_LeafLooper_02():
    container = abjad.Container(r"c'4 d'2 e'4 f'2 ~ f'8 g'4.")
    looper = auxjad.LeafLooper(container,
                               window_size=3,
                               step_size=1,
                               max_steps=2,
                               repetition_chance=0.25,
                               forward_bias=0.2,
                               head_position=0,
                               end_with_max_n_leaves=True,
                               omit_time_signatures=False,
                               process_on_first_call=True,
                               )
    assert looper.window_size == 3
    assert looper.step_size == 1
    assert looper.max_steps == 2
    assert looper.repetition_chance == 0.25
    assert looper.forward_bias == 0.2
    assert looper.head_position == 0
    assert looper.end_with_max_n_leaves
    assert not looper.omit_time_signatures
    assert looper.process_on_first_call
    looper.window_size = 2
    looper.step_size = 2
    looper.max_steps = 3
    looper.repetition_chance = 0.1
    looper.forward_bias = 0.8
    looper.head_position = 2
    looper.end_with_max_n_leaves = False
    looper.omit_time_signatures = True
    looper.process_on_first_call = False
    assert looper.window_size == 2
    assert looper.step_size == 2
    assert looper.max_steps == 3
    assert looper.repetition_chance == 0.1
    assert looper.forward_bias == 0.8
    assert looper.head_position == 2
    assert not looper.end_with_max_n_leaves
    assert looper.omit_time_signatures
    assert not looper.process_on_first_call


def test_LeafLooper_03():
    container = abjad.Container(r"c'4 d'2 e'4 f'2 ~ f'8 g'4.")
    looper = auxjad.LeafLooper(container, window_size=3)
    assert looper.head_position == 0
    looper()
    assert looper.head_position == 0
    looper()
    assert looper.head_position == 1
    looper()
    assert looper.head_position == 2


def test_LeafLooper_04():
    container = abjad.Container(r"c'4 d'2 e'4 f'2 ~ f'8 g'4.")
    looper = auxjad.LeafLooper(container, window_size=3)
    assert len(looper) == 5


def test_LeafLooper_05():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    looper = auxjad.LeafLooper(container, window_size=2)
    notes = looper.output_all()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 2/4
            c'4
            d'4
            d'4
            e'4
            e'4
            f'4
            \time 1/4
            f'4
        }
        """
    )


def test_LeafLooper_06():
    container = abjad.Container(r"c'4 d'4 e'8 f'4 g'2")
    looper = auxjad.LeafLooper(container, window_size=3)
    notes = looper()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 5/8
            c'4
            d'8
            ~
            d'8
            e'8
        }
        """
    )
    looper.window_size = 4
    notes = looper()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 9/8
            d'4
            e'8
            f'4
            g'2
        }
        """
    )


def test_LeafLooper_07():
    container = abjad.Container(r"c'4 d'16 e'8 f'2 g'8")
    looper = auxjad.LeafLooper(container,
                               window_size=3,
                               head_position=2,
                               )
    notes = looper()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            e'8
            f'8
            ~
            f'4.
            g'8
        }
        """
    )


def test_LeafLooper_08():
    container = abjad.Container(r"c'4 d'8 \times 2/3 {a4 g2}")
    looper = auxjad.LeafLooper(container, window_size=2)
    notes = looper.output_all()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/8
            c'4
            d'8
            #(ly:expect-warning "strange time signature found")
            \time 7/24
            d'8
            \tweak edge-height #'(0.7 . 0)
            \times 2/3
            {
                a4
            }
            \times 2/3
            {
                \time 2/4
                a4
                g2
            }
            \tweak edge-height #'(0.7 . 0)
            \times 2/3
            {
                #(ly:expect-warning "strange time signature found")
                \time 2/6
                g2
            }
        }
        """
    )


def test_LeafLooper_09():
    container = abjad.Container(r"c'4 d'2 e'8")
    looper = auxjad.LeafLooper(container, window_size=2)
    notes = looper.__next__()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'4
            d'2
        }
        """
    )
    notes = looper.__next__()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 5/8
            d'2
            e'8
        }
        """
    )
    notes = looper.__next__()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 1/8
            e'8
        }
        """
    )
    with pytest.raises(StopIteration):
        notes = looper.__next__()  # noqa: F841


def test_LeafLooper_10():
    container = abjad.Container(r"c'4 d'2 e'4 f'2 ~ f'8 g'4.")
    looper = auxjad.LeafLooper(container,
                               window_size=3,
                               omit_time_signatures=True,
                               )
    notes = looper()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            d'2
            e'4
        }
        """
    )


def test_LeafLooper_11():
    wrong_type_input = 'foo'
    container = abjad.Container(r"c'4 d'2 e'4 f'2 ~ f'8 g'4.")
    with pytest.raises(TypeError):
        looper = auxjad.LeafLooper(wrong_type_input,  # noqa: F841
                                   window_size=3,
                                   )
        looper = auxjad.LeafLooper(container,  # noqa: F841
                                   window_size='foobar',
                                   )
        looper = auxjad.LeafLooper(container,  # noqa: F841
                                   window_size=3,
                                   step_size='foobar',
                                   )
        looper = auxjad.LeafLooper(container,  # noqa: F841
                                   window_size=3,
                                   max_steps='foobar',
                                   )
        looper = auxjad.LeafLooper(container,  # noqa: F841
                                   window_size=3,
                                   repetition_chance='foobar',
                                   )
        looper = auxjad.LeafLooper(container,  # noqa: F841
                                   window_size=3,
                                   head_position='foobar',
                                   )
        looper = auxjad.LeafLooper(container,  # noqa: F841
                                   window_size=3,
                                   omit_time_signatures='foobar',
                                   )
    with pytest.raises(ValueError):
        looper = auxjad.LeafLooper(container, window_size=100)  # noqa: F841
        looper = auxjad.LeafLooper(container,  # noqa: F841
                                   window_size=3,
                                   step_size=-1,
                                   )
        looper = auxjad.LeafLooper(container,  # noqa: F841
                                   window_size=3,
                                   step_size=100,
                                   )
        looper = auxjad.LeafLooper(container,  # noqa: F841
                                   window_size=3,
                                   max_steps=-1,
                                   )
        looper = auxjad.LeafLooper(container,  # noqa: F841
                                   window_size=3,
                                   repetition_chance=-0.3,
                                   )
        looper = auxjad.LeafLooper(container,  # noqa: F841
                                   window_size=3,
                                   repetition_chance=1.4,
                                   )
        looper = auxjad.LeafLooper(container,  # noqa: F841
                                   window_size=3,
                                   head_position=-1,
                                   )
        looper = auxjad.LeafLooper(container,  # noqa: F841
                                   window_size=3,
                                   head_position=100,
                                   )


def test_LeafLooper_12():
    container = abjad.Container(r"c'4 d'2 r8 d'4 <e' g'>8 r4 f'2. <e' g'>16")
    looper = auxjad.LeafLooper(container,
                               window_size=4,
                               disable_rewrite_meter=True,
                               )
    notes = looper.output_all(tie_identical_pitches=True)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 9/8
            c'4
            d'2
            r8
            d'4
            ~
            \time 4/4
            d'2
            r8
            d'4
            <e' g'>8
            \time 3/4
            r8
            d'4
            <e' g'>8
            r4
            \time 11/8
            d'4
            <e' g'>8
            r4
            f'2.
            \time 19/16
            <e' g'>8
            r4
            f'2.
            <e' g'>16
            \time 17/16
            r4
            f'2.
            <e' g'>16
            \time 13/16
            f'2.
            <e' g'>16
            ~
            \time 1/16
            <e' g'>16
        }
        """
    )


def test_LeafLooper_13():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    looper = auxjad.LeafLooper(container, window_size=2)
    notes = looper.output_n(2)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 2/4
            c'4
            d'4
            d'4
            e'4
        }
        """
    )


def test_LeafLooper_14():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    looper = auxjad.LeafLooper(container, window_size=2)
    notes = looper.output_n(2, tie_identical_pitches=True)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 2/4
            c'4
            d'4
            ~
            d'4
            e'4
        }
        """
    )


def test_LeafLooper_15():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    looper = auxjad.LeafLooper(container, window_size=2)
    with pytest.raises(RuntimeError):
        notes = looper.output_n(100)  # noqa: F841


def test_LeafLooper_16():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    looper = auxjad.LeafLooper(container,
                               window_size=2,
                               head_position=2,
                               forward_bias=0.0,
                               )
    notes = looper.output_all()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 2/4
            e'4
            f'4
            d'4
            e'4
            c'4
            d'4
        }
        """
    )


def test_LeafLooper_17():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    looper = auxjad.LeafLooper(container,
                               window_size=2,
                               head_position=0,
                               forward_bias=0.0,
                               )
    notes = looper.output_all()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 2/4
            c'4
            d'4
        }
        """
    )


def test_LeafLooper_18():
    container = abjad.Container(r"c'4 d'8 e'2 f'8 g'4")
    looper = auxjad.LeafLooper(container,
                               window_size=3,
                               process_on_first_call=True,
                               )
    notes = looper()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            d'8
            e'8
            ~
            e'4.
            f'8
        }
        """
    )


def test_LeafLooper_19():
    container = abjad.Container(r"c'4 d'4 e'4 f'4 g'4 a'4")
    looper = auxjad.LeafLooper(container,
                               window_size=3,
                               )
    notes = looper()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'4
            d'4
            e'4
        }
        """
    )
    notes = looper()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            d'4
            e'4
            f'4
        }
        """
    )
    looper.contents = abjad.Container(r"cs'''4 ds'''4 es'''4 fs'''4")
    notes = looper()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            ds'''4
            es'''4
            fs'''4
        }
        """
    )
    looper.head_position = 0
    notes = looper()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            cs'''4
            ds'''4
            es'''4
        }
        """
    )


def test_LeafLooper_20():
    random.seed(15231)
    container = abjad.Container(r"c'4 d'4 e'4 f'4 g'4 a'4 b'4 c''4")
    looper = auxjad.LeafLooper(container,
                               window_size=3,
                               head_position=3,
                               forward_bias=0.5,
                               )
    notes = looper.output_n(5)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            f'4
            g'4
            a'4
            e'4
            f'4
            g'4
            f'4
            g'4
            a'4
            e'4
            f'4
            g'4
            d'4
            e'4
            f'4
        }
        """
    )


def test_LeafLooper_21():
    random.seed(55126)
    container = abjad.Container(r"c'4 d'4 e'4 f'4 g'4 a'4 b'4 c''4 d''4")
    looper = auxjad.LeafLooper(container,
                               window_size=2,
                               max_steps=4,
                               )
    notes = looper.output_n(4)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 2/4
            c'4
            d'4
            g'4
            a'4
            a'4
            b'4
            c''4
            d''4
        }
        """
    )


def test_LeafLooper_22():
    container = abjad.Container(r"c'4 d'2 e'8 f'2")
    looper = auxjad.LeafLooper(container,
                               window_size=2,
                               )
    staff = abjad.Staff()
    for window in looper:
        staff.append(window)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'4
            d'2
            \time 5/8
            d'2
            e'8
            \time 5/8
            e'8
            f'2
            \time 2/4
            f'2
        }
        """
    )
    auxjad.mutate.remove_repeated_time_signatures(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            c'4
            d'2
            \time 5/8
            d'2
            e'8
            e'8
            f'2
            \time 2/4
            f'2
        }
        """
    )


def test_LeafLooper_23():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    looper = auxjad.LeafLooper(container, window_size=2)
    assert isinstance(looper(), abjad.Selection)
    tuplet = abjad.Tuplet('3:2', r"c'2 d'2 e'2")
    looper = auxjad.LeafLooper(tuplet, window_size=2)
    assert isinstance(looper(), abjad.Selection)
    voice = abjad.Voice(r"c'4 d'4 e'4 f'4")
    looper = auxjad.LeafLooper(voice, window_size=2)
    assert isinstance(looper(), abjad.Selection)
    staff = abjad.Staff(r"c'4 d'4 e'4 f'4")
    looper = auxjad.LeafLooper(staff, window_size=2)
    assert isinstance(looper(), abjad.Selection)
    score = abjad.Score([abjad.Staff(r"c'4 d'4 e'4 f'4")])
    looper = auxjad.LeafLooper(score, window_size=2)
    assert isinstance(looper(), abjad.Selection)
    voice = abjad.Voice(r"c'4 d'4 e'4 f'4")
    staff = abjad.Staff([voice])
    looper = auxjad.LeafLooper(staff, window_size=2)
    assert isinstance(looper(), abjad.Selection)
    staff = abjad.Staff(r"c'4 d'4 e'4 f'4")
    score = abjad.Score([staff])
    looper = auxjad.LeafLooper(score, window_size=2)
    assert isinstance(looper(), abjad.Selection)

    voice1 = abjad.Voice(r"c'4 d'4 e'4 f'4")
    voice2 = abjad.Voice(r"g2 f2")
    staff = abjad.Staff([voice1, voice2], simultaneous=True)
    with pytest.raises(ValueError):
        looper = auxjad.LeafLooper(staff, window_size=2)  # noqa: F841

    staff1 = abjad.Staff(r"c'4 d'4 e'4 f'4")
    staff2 = abjad.Staff(r"g2 f2")
    score = abjad.Score([staff1, staff2])
    with pytest.raises(ValueError):
        looper = auxjad.LeafLooper(score, window_size=2)  # noqa: F841


def test_LeafLooper_24():
    container = abjad.Container(r"c'4\p( d'2 e'4\f) f'2( ~ f'8 g'4 a'1\pp)")
    looper = auxjad.LeafLooper(container,
                               window_size=3,
                               disable_rewrite_meter=True,
                               )
    notes = looper.output_n(5)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            c'4
            \p
            (
            d'2
            e'4
            \f
            )
            \time 11/8
            d'2
            \p
            (
            e'4
            \f
            )
            f'2
            ~
            f'8
            \time 9/8
            e'4
            f'2
            (
            ~
            f'8
            g'4
            )
            \time 15/8
            f'2
            (
            ~
            f'8
            g'4
            a'1
            \pp
            )
            \time 5/4
            g'4
            \f
            (
            a'1
            \pp
            )
        }
        """
    )


def test_LeafLooper_25():
    container = abjad.Container(r"c'16 d'4 e'8. f'4 g'16")
    looper = auxjad.LeafLooper(container,
                               window_size=3,
                               )
    notes = looper.output_n(3)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 2/4
            c'16
            d'8.
            ~
            d'16
            e'8.
            \time 11/16
            d'4
            e'8.
            f'4
            \time 2/4
            e'8.
            f'16
            ~
            f'8.
            g'16
        }
        """
    )
    looper = auxjad.LeafLooper(container,
                               window_size=3,
                               disable_rewrite_meter=True,
                               )
    notes = looper.output_n(3)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 2/4
            c'16
            d'4
            e'8.
            \time 11/16
            d'4
            e'8.
            f'4
            \time 2/4
            e'8.
            f'4
            g'16
        }
        """
    )


def test_LeafLooper_26():
    container = abjad.Container(r"c'4 d'4. e'4 f'8")
    looper = auxjad.LeafLooper(container, window_size=3)
    notes = looper.output_all()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 7/8
            c'4
            d'4.
            e'4
            \time 3/4
            d'4.
            e'4
            f'8
            \time 3/8
            e'4
            f'8
            \time 1/8
            f'8
        }
        """
    )
    container = abjad.Container(r"c'4 d'4. e'4 f'8")
    looper = auxjad.LeafLooper(container,
                               window_size=3,
                               end_with_max_n_leaves=True,
                               )
    notes = looper.output_all()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 7/8
            c'4
            d'4.
            e'4
            \time 3/4
            d'4.
            e'4
            f'8
        }
        """
    )



import random

import auxjad


def test_GeneticAlgorithm_01():
    ga = auxjad.GeneticAlgorithm(
        target=['A', 'B', 'C', 'D', 'E'],
        genes=['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J'],
    )
    assert format(ga) == "['A', 'B', 'C', 'D', 'E']"
    assert len(ga) == 5
    assert ga.initial_individual is None
    assert ga.population_size == 100
    assert ga.select_n_parents == 10
    assert ga.mutation_chance == 0.2
    assert ga.mutation_index == 0.1
    assert ga.evaluation_index == 0.2
    assert ga.target == ['A', 'B', 'C', 'D', 'E']


def test_GeneticAlgorithm_02():
    random.seed(87124)
    ga = auxjad.GeneticAlgorithm(
        target=['A', 'B', 'C', 'D', 'E'],
        genes=['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J'],
    )
    assert ga.generation_number is None
    ga()
    assert ga.generation_number == 0
    assert ga.fittest_individual == ['D', 'C', 'A', 'D', 'E']
    ga()
    assert ga.generation_number == 1
    assert ga.fittest_individual == ['A', 'C', 'A', 'D', 'E']
    ga()
    assert ga.generation_number == 2
    assert ga.fittest_individual == ['A', 'C', 'C', 'D', 'E']


def test_GeneticAlgorithm_03():
    random.seed(55127)
    ga = auxjad.GeneticAlgorithm(
        target=['A', 'B', 'C', 'D', 'E'],
        genes=['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J'],
        population_size=4,
        select_n_parents=2,
    )
    ga()
    assert ga.population == [['A', 'J', 'J', 'B', 'H'],
                             ['D', 'A', 'E', 'A', 'F'],
                             ['F', 'F', 'A', 'F', 'F'],
                             ['F', 'F', 'E', 'J', 'C'],
                             ]
    assert ga.scores == [0.209603072,
                         0.0912,
                         0.05638400000000001,
                         0.016396800000000003,
                         ]


def test_GeneticAlgorithm_04():
    random.seed(60012)
    ga = auxjad.GeneticAlgorithm(
        target=['A', 'B', 'C', 'D', 'E'],
        genes=['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J'],
        population_size=50,
    )
    expected_results = [['A', 'J', 'B', 'E', 'E'],
                        ['A', 'H', 'C', 'D', 'J'],
                        ['A', 'C', 'D', 'D', 'E'],
                        ['A', 'C', 'D', 'D', 'E'],
                        ['A', 'C', 'D', 'D', 'E'],
                        ['A', 'C', 'C', 'D', 'E'],
                        ['A', 'C', 'C', 'D', 'E'],
                        ['A', 'C', 'C', 'D', 'E'],
                        ['A', 'C', 'C', 'D', 'E'],
                        ['A', 'C', 'C', 'D', 'E'],
                        ]
    expected_scores = [0.480000512,
                       0.6000768,
                       0.6799999999999999,
                       0.6799999999999999,
                       0.6799999999999999,
                       0.8400000000000001,
                       0.8400000000000001,
                       0.8400000000000001,
                       0.8400000000000001,
                       0.8400000000000001,
                       ]
    for i in range(10):
        ga()
        assert ga.generation_number == i
        assert ga.fittest_individual == expected_results[i]
        assert ga.fittest_individual_score == expected_scores[i]


def test_GeneticAlgorithm_05():
    random.seed(51902)
    ga = auxjad.GeneticAlgorithm(
        target=['A', 'B', 'C', 'D', 'E'],
        genes=['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J'],
    )
    assert ga.fittest_individual is None
    assert ga.generation_number is None
    for _ in range(10):
        ga()
    assert ga.fittest_individual == ['A', 'B', 'C', 'D', 'E']
    assert ga.generation_number == 9
    ga.reset()
    assert ga.fittest_individual is None
    assert ga.generation_number is None


def test_GeneticAlgorithm_06():
    random.seed(51244)
    ga = auxjad.GeneticAlgorithm(
        target=['A', 'B', 'C', 'D', 'E'],
        genes=['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J'],
        initial_individual=['F', 'G', 'H', 'I', 'J'],
    )
    expected_results = [['F', 'G', 'H', 'I', 'J'],
                        ['F', 'B', 'H', 'I', 'J'],
                        ['F', 'B', 'H', 'I', 'E'],
                        ['F', 'B', 'H', 'I', 'E'],
                        ['A', 'B', 'H', 'I', 'E'],
                        ['A', 'B', 'E', 'I', 'E'],
                        ['A', 'B', 'E', 'D', 'E'],
                        ['A', 'B', 'E', 'D', 'E'],
                        ['A', 'B', 'E', 'D', 'E'],
                        ['A', 'B', 'D', 'D', 'E'],
                        ]
    for expected_result in expected_results:
        ga()
        assert ga.fittest_individual == expected_result


def test_GeneticAlgorithm_07():
    random.seed(51244)
    ga = auxjad.GeneticAlgorithm(
        target=['A', 'B', 'C', 'D', 'E'],
        genes=['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J'],
    )
    assert ga.population_size == 100
    assert ga.select_n_parents == 10
    assert ga.keep_n_parents == 0
    assert ga.mutation_chance == 0.2
    assert ga.mutation_index == 0.1
    assert ga.evaluation_index == 0.2
    ga.population_size = 50
    ga.select_n_parents = 15
    ga.keep_n_parents = 3
    ga.mutation_chance = 0.75
    ga.mutation_index = 0.25
    ga.evaluation_index = 0.5
    assert ga.population_size == 50
    assert ga.select_n_parents == 15
    assert ga.keep_n_parents == 3
    assert ga.mutation_chance == 0.75
    assert ga.mutation_index == 0.25
    assert ga.evaluation_index == 0.5


def test_GeneticAlgorithm_08():
    random.seed(89226)
    ga = auxjad.GeneticAlgorithm(
        target=['A', 'B', 'C', 'D', 'E'],
        genes=['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J'],
    )
    ga.__next__()
    assert ga.fittest_individual == ['A', 'C', 'D', 'D', 'D']
    ga.__next__()
    assert ga.fittest_individual == ['A', 'C', 'E', 'D', 'E']
    next(ga)
    assert ga.fittest_individual == ['A', 'C', 'E', 'D', 'E']
    next(ga)
    assert ga.fittest_individual == ['A', 'C', 'D', 'D', 'E']



import random

import abjad
import pytest

import auxjad


def test_PitchRandomiser_01():
    random.seed(16284)
    container = abjad.Container(r"\time 4/4 c'4 d'4 e'4 f'4")
    pitches = r"fs' gs' a' b' cs''"
    randomiser = auxjad.PitchRandomiser(container,
                                        pitches,
                                        )
    assert format(randomiser) == "PitchSegment(\"fs' gs' a' b' cs''\")"
    notes = randomiser()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            a'4
            b'4
            cs''4
            fs'4
        }
        """
    )
    notes = randomiser()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            b'4
            b'4
            b'4
            fs'4
        }
        """
    )
    notes = randomiser.current_window
    with pytest.raises(AttributeError):
        randomiser.current_window = abjad.Container(r"c''2 e''2")
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 4/4
            b'4
            b'4
            b'4
            fs'4
        }
        """
    )


def test_PitchRandomiser_02():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    randomiser = auxjad.PitchRandomiser(container,
                                        pitches=r"a b cs' ds' e'",
                                        weights=[1.0, 2.0, 1.0, 1.5, 1.3],
                                        omit_time_signatures=True,
                                        process_on_first_call=True,
                                        use_tenney_selector=True,
                                        )
    assert randomiser.pitches == abjad.PitchSegment(r"a b cs' ds' e'")
    assert randomiser.weights == [1.0, 2.0, 1.0, 1.5, 1.3]
    assert randomiser.omit_time_signatures
    assert randomiser.process_on_first_call
    assert randomiser.use_tenney_selector
    randomiser.pitches = abjad.PitchSegment(r"c' d' e' f'")
    randomiser.weights = [1, 2, 5, 8]
    randomiser.omit_time_signatures = False
    randomiser.process_on_first_call = False
    randomiser.use_tenney_selector = False
    assert randomiser.pitches == abjad.PitchSegment(r"c' d' e' f'")
    assert randomiser.weights == [1, 2, 5, 8]
    assert not randomiser.omit_time_signatures
    assert not randomiser.process_on_first_call
    assert not randomiser.use_tenney_selector


def test_PitchRandomiser_03():
    random.seed(19387)
    container = abjad.Container(r"c'8. d'4 r8 r8. e'16 f'8.")
    pitches = [6, 7, 8, 9, 10, 11]
    randomiser = auxjad.PitchRandomiser(container,
                                        pitches,
                                        )
    notes = randomiser()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            bf'8.
            af'4
            r8
            r8.
            bf'16
            a'8.
        }
        """
    )


def test_PitchRandomiser_04():
    random.seed(87612)
    container = abjad.Container(r"<c' e' g'>8. d'4 r8 r8. e'16 <f' a'>8.")
    pitches = [6, 7, 8, 9, 10, 11]
    randomiser = auxjad.PitchRandomiser(container,
                                        pitches,
                                        )
    notes = randomiser()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            <fs' g' bf'>8.
            b'4
            r8
            r8.
            bf'16
            <fs' bf'>8.
        }
        """
    )


def test_PitchRandomiser_05():
    random.seed(31987)
    container = abjad.Container(r"<c' e' g' a'>2 <cs' ds' e' f' g' a' b'>2")
    pitches = [6, 7, 8]
    randomiser = auxjad.PitchRandomiser(container,
                                        pitches,
                                        )
    notes = randomiser()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            <fs' g' af'>2
            <fs' g' af'>2
        }
        """
    )


def test_PitchRandomiser_06():
    random.seed(87132)
    container = abjad.Container(r"c'8 d'8 e'8 f'8 g'8 a'8 b'8 c'8")
    pitches = r"fs' gs' a' b'"
    randomiser = auxjad.PitchRandomiser(container,
                                        pitches,
                                        use_tenney_selector=True,
                                        )
    notes = randomiser()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            fs'8
            a'8
            fs'8
            gs'8
            a'8
            b'8
            gs'8
            fs'8
        }
        """
    )


def test_PitchRandomiser_07():
    random.seed(67612)
    container = abjad.Container(r"c'8 d'8 e'8 f'8 g'8 a'8 b'8 c'8")
    pitches = r"fs' gs' a' b'"
    randomiser = auxjad.PitchRandomiser(container,
                                        pitches,
                                        weights=[5.0, 2.0, 1.5, 1.0],
                                        )
    notes = randomiser()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            b'8
            fs'8
            gs'8
            gs'8
            gs'8
            fs'8
            fs'8
            a'8
        }
        """
    )


def test_PitchRandomiser_08():
    random.seed(81223)
    container = abjad.Container(r"c'8 d'8 e'8 f'8 g'8 a'8 b'8 c'8")
    pitches = r"fs' gs' a' b'"
    randomiser = auxjad.PitchRandomiser(container,
                                        pitches,
                                        weights=[5.0, 2.0, 1.5, 1.0],
                                        use_tenney_selector=True,
                                        )
    notes = randomiser()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            fs'8
            gs'8
            a'8
            fs'8
            gs'8
            a'8
            gs'8
            b'8
        }
        """
    )


def test_PitchRandomiser_09():
    random.seed(91445)
    container = abjad.Container(r"c'8 d'8 e'8 f'8 g'8 a'8 b'8 c'8")
    pitches = r"fs' gs' a' b'"
    randomiser = auxjad.PitchRandomiser(container,
                                        pitches,
                                        weights=[100.0, 1.0, 1.0, 1.0],
                                        )
    notes = randomiser()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            fs'8
            fs'8
            fs'8
            fs'8
            fs'8
            fs'8
            fs'8
            fs'8
        }
        """
    )
    randomiser.weights = None
    notes = randomiser()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            fs'8
            fs'8
            b'8
            gs'8
            gs'8
            a'8
            fs'8
            b'8
        }
        """
    )


def test_PitchRandomiser_10():
    container = abjad.Container(r"c'8 d'8 e'8 f'8 g'8 a'8 b'8 c'8")
    pitches = r"fs' gs' a' b'"
    randomiser = auxjad.PitchRandomiser(container,
                                        pitches,
                                        weights=[100.0, 1.0, 1.0, 1.0],
                                        )
    randomiser.pitches = r"c'' d'' e'' f''"
    assert randomiser.pitches == abjad.PitchSegment(r"c'' d'' e'' f''")
    assert randomiser.weights == [100.0, 1.0, 1.0, 1.0]


def test_PitchRandomiser_11():
    container = abjad.Container(r"c'8 d'8 e'8 f'8 g'8 a'8 b'8 c'8")
    pitches = r"fs' gs' a' b'"
    randomiser = auxjad.PitchRandomiser(container,
                                        pitches,
                                        weights=[100.0, 1.0, 1.0, 1.0],
                                        )
    randomiser.pitches = r"c'' d'' e'' f'' g'' a'' b''"
    assert randomiser.pitches == abjad.PitchSegment(
        r"c'' d'' e'' f'' g'' a'' b''"
    )
    assert randomiser.weights is None


def test_PitchRandomiser_12():
    random.seed(95877)
    container = abjad.Container(r"c'4 ~ c'16 r8. d'4 e'8. r16")
    pitches = [6, 7, 8, 9, 10]
    randomiser = auxjad.PitchRandomiser(container,
                                        pitches,
                                        )
    notes = randomiser.output_n(3)
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            a'4
            ~
            a'16
            r8.
            g'4
            fs'8.
            r16
            g'4
            ~
            g'16
            r8.
            g'4
            fs'8.
            r16
            bf'4
            ~
            bf'16
            r8.
            a'4
            af'8.
            r16
        }
        """
    )


def test_PitchRandomiser_13():
    random.seed(88112)
    container = abjad.Container(
        r"c'4\p\< ~ c'8. d'16-.\f e'4--\pp f'8.( g'16)"
    )
    pitches = [6, 7, 8, 9, 10, 11, 12]
    randomiser = auxjad.PitchRandomiser(container,
                                        pitches,
                                        )
    notes = randomiser()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            a'4
            \p
            \<
            ~
            a'8.
            c''16
            \f
            - \staccato
            af'4
            \pp
            - \tenuto
            a'8.
            (
            bf'16
            )
        }
        """
    )


def test_PitchRandomiser_14():
    random.seed(97112)
    container = abjad.Container(r"\time 3/4 c'4 d'2 \time 2/4 e'8 f'8 g'8 a'8")
    pitches = r"fs' gs' a' b'"
    randomiser = auxjad.PitchRandomiser(container,
                                        pitches,
                                        )
    notes = randomiser()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            a'4
            fs'2
            \time 2/4
            gs'8
            gs'8
            a'8
            gs'8
        }
        """
    )


def test_PitchRandomiser_15():
    random.seed(97112)
    container = abjad.Container(r"\time 3/4 c'4 d'2 \time 2/4 e'8 f'8 g'8 a'8")
    pitches = r"fs' gs' a' b'"
    randomiser = auxjad.PitchRandomiser(container,
                                        pitches,
                                        omit_time_signatures=True,
                                        )
    notes = randomiser()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            a'4
            fs'2
            gs'8
            gs'8
            a'8
            gs'8
        }
        """
    )


def test_PitchRandomiser_16():
    random.seed(45017)
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    pitches = r"fs' gs' a' b'"
    randomiser = auxjad.PitchRandomiser(container,
                                        pitches,
                                        process_on_first_call=False,
                                        )
    notes = randomiser()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            d'4
            e'4
            f'4
        }
        """
    )
    notes = randomiser()
    staff = abjad.Staff(notes)
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            b'4
            fs'4
            gs'4
            fs'4
        }
        """
    )


def test_PitchRandomiser_17():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    pitches = r"fs' gs' a' b'"
    weights = None
    assert auxjad.PitchRandomiser(container, pitches, weights=weights)
    weights = [1, 1, 5, 2]
    assert auxjad.PitchRandomiser(container, pitches, weights=weights)
    weights = [1, 1, 5, 2, 3, 4, 8]
    with pytest.raises(ValueError):
        randomiser = auxjad.PitchRandomiser(container,  # noqa: F841
                                            pitches,
                                            weights=weights,
                                            )


def test_PitchRandomiser_18():
    random.seed(91773)
    container = abjad.Container(r"\time 3/4 c'4 d'4 e'4")
    pitches = r"fs' gs' a' b' cs''"
    randomiser = auxjad.PitchRandomiser(container,
                                        pitches,
                                        )
    staff = abjad.Staff()
    for window in randomiser:
        staff.append(window)
        if abjad.get.duration(staff) == abjad.Duration((9, 4)):
            break
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            a'4
            b'4
            gs'4
            \time 3/4
            fs'4
            a'4
            b'4
            \time 3/4
            a'4
            gs'4
            cs''4
        }
        """
    )
    auxjad.mutate.remove_repeated_time_signatures(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            \time 3/4
            a'4
            b'4
            gs'4
            fs'4
            a'4
            b'4
            a'4
            gs'4
            cs''4
        }
        """
    )


def test_PitchRandomiser_19():
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    pitches = r"fs' gs' a' b'"
    randomiser = auxjad.PitchRandomiser(container,
                                        pitches,
                                        )
    assert len(randomiser) == 4
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    pitches = [6, 7, 8, 9, 10, 11, 12]
    randomiser = auxjad.PitchRandomiser(container,
                                        pitches,
                                        )
    assert len(randomiser) == 7


def test_PitchRandomiser_20():
    pitches = r"fs' gs' a' b' cs''"
    container = abjad.Container(r"c'4 d'4 e'4 f'4")
    randomiser = auxjad.PitchRandomiser(container, pitches)
    assert isinstance(randomiser(), abjad.Selection)
    tuplet = abjad.Tuplet('3:2', r"c'2 d'2 e'2")
    randomiser = auxjad.PitchRandomiser(tuplet, pitches)
    assert isinstance(randomiser(), abjad.Selection)
    voice = abjad.Voice(r"c'4 d'4 e'4 f'4")
    randomiser = auxjad.PitchRandomiser(voice, pitches)
    assert isinstance(randomiser(), abjad.Selection)
    staff = abjad.Staff(r"c'4 d'4 e'4 f'4")
    randomiser = auxjad.PitchRandomiser(staff, pitches)
    assert isinstance(randomiser(), abjad.Selection)
    score = abjad.Score([abjad.Staff(r"c'4 d'4 e'4 f'4")])
    randomiser = auxjad.PitchRandomiser(score, pitches)
    assert isinstance(randomiser(), abjad.Selection)
    voice = abjad.Voice(r"c'4 d'4 e'4 f'4")
    staff = abjad.Staff([voice])
    randomiser = auxjad.PitchRandomiser(staff, pitches)
    assert isinstance(randomiser(), abjad.Selection)
    staff = abjad.Staff(r"c'4 d'4 e'4 f'4")
    score = abjad.Score([staff])
    randomiser = auxjad.PitchRandomiser(score, pitches)
    assert isinstance(randomiser(), abjad.Selection)

    voice1 = abjad.Voice(r"c'4 d'4 e'4 f'4")
    voice2 = abjad.Voice(r"g2 f2")
    staff = abjad.Staff([voice1, voice2], simultaneous=True)
    with pytest.raises(ValueError):
        randomiser = auxjad.PitchRandomiser(staff, pitches)  # noqa: F841

    staff1 = abjad.Staff(r"c'4 d'4 e'4 f'4")
    staff2 = abjad.Staff(r"g2 f2")
    score = abjad.Score([staff1, staff2])
    with pytest.raises(ValueError):
        randomiser = auxjad.PitchRandomiser(score, pitches)  # noqa: F841



import abjad

import auxjad


def test_merge_hairpins_01():
    staff = abjad.Staff(r"c'4\pp\< d'4 e'4\p\< f'4 g'1\mp")
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            \pp
            \<
            d'4
            e'4
            \p
            \<
            f'4
            g'1
            \mp
        }
        """
    )
    auxjad.mutate.merge_hairpins(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            \pp
            \<
            d'4
            e'4
            f'4
            g'1
            \mp
        }
        """
    )


def test_merge_hairpins_02():
    staff = abjad.Staff(r"c'4\ff\> d'4 e'4\mf\> f'4 g'1\pp")
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            \ff
            \>
            d'4
            e'4
            \mf
            \>
            f'4
            g'1
            \pp
        }
        """
    )
    auxjad.mutate.merge_hairpins(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            \ff
            \>
            d'4
            e'4
            f'4
            g'1
            \pp
        }
        """
    )


def test_merge_hairpins_03():
    staff = abjad.Staff(r"c'4\pp\< d'4 e'4\mf\> f'4 g'1\pp")
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            \pp
            \<
            d'4
            e'4
            \mf
            \>
            f'4
            g'1
            \pp
        }
        """
    )
    auxjad.mutate.merge_hairpins(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            \pp
            \<
            d'4
            e'4
            \mf
            \>
            f'4
            g'1
            \pp
        }
        """
    )


def test_merge_hairpins_04():
    staff = abjad.Staff(
        r"c'1\pp\< d'2\f d'2\< e'1\ff f'1\f\> g'2\mp g'2\> a'1\! b'1\ppp"
    )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            \pp
            \<
            d'2
            \f
            d'2
            \<
            e'1
            \ff
            f'1
            \f
            \>
            g'2
            \mp
            g'2
            \>
            a'1
            \!
            b'1
            \ppp
        }
        """
    )
    auxjad.mutate.merge_hairpins(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'1
            \pp
            \<
            d'2
            \f
            d'2
            \<
            e'1
            \ff
            f'1
            \f
            \>
            g'2
            \mp
            g'2
            \>
            a'1
            \!
            b'1
            \ppp
        }
        """
    )


def test_merge_hairpins_05():
    staff = abjad.Staff(
        r"c'4\pp\< d'4 e'4\p\< f'4 g'4\mp\< a'4 b'4\mf\< c''4"
        r"d''4\f\> c''4 b'4\mf\> a'4 g'4\mp\> f'4 e'4\p\> d'4"
        r"c'1\pp"
    )
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            \pp
            \<
            d'4
            e'4
            \p
            \<
            f'4
            g'4
            \mp
            \<
            a'4
            b'4
            \mf
            \<
            c''4
            d''4
            \f
            \>
            c''4
            b'4
            \mf
            \>
            a'4
            g'4
            \mp
            \>
            f'4
            e'4
            \p
            \>
            d'4
            c'1
            \pp
        }
        """
    )
    auxjad.mutate.merge_hairpins(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            \pp
            \<
            d'4
            e'4
            f'4
            g'4
            a'4
            b'4
            c''4
            d''4
            \f
            \>
            c''4
            b'4
            a'4
            g'4
            f'4
            e'4
            d'4
            c'1
            \pp
        }
        """
    )


def test_merge_hairpins_06():
    staff = abjad.Staff(r"c'4\pp\< d'4 e'4\p\> f'4 g'1\pp")
    abjad.mutate.merge_hairpins(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            \pp
            \<
            d'4
            e'4
            \p
            \>
            f'4
            g'1
            \pp
        }
        """
    )


def test_merge_hairpins_07():
    staff = abjad.Staff(r"c'4\pp\< d'4 e'4\p\< f'4 g'1\mp")
    abjad.mutate.merge_hairpins(staff[:])
    assert abjad.lilypond(staff) == abjad.String.normalize(
        r"""
        \new Staff
        {
            c'4
            \pp
            \<
            d'4
            e'4
            f'4
            g'1
            \mp
        }
        """
    )
